import React from 'react';
import ReactDom from 'react-dom';
import { Router, Route, Switch } from 'dva/router';
import Markdown from './pages/Markdown';

const cacheScript = {};

function loadComponent(scope, module) {
  return async () => {
    console.log('----module ', scope, module);
    // Initializes the share scope. This fills it with known provided modules from this build and all remotes
    await __webpack_init_sharing__('default');
    const container = window[scope];
    console.log('----module container', container); // or get the container somewhere else
    // Initialize the container, it may provide shared modules
    await container.init(__webpack_share_scopes__.default);
    console.log('----module container', 2);
    const factory = await window[scope].get(module);
    console.log('----module factory', factory);
    const Module = factory();
    console.log('----module ', Module);
    return Module.default || Module;
  };
}

const useDynamicScript = args => {
  if (!args.url) {
    return;
  }

  return new Promise((resolve, reject) => {
    if (cacheScript[args.url]) {
      resolve(true);
      return;
    }
    const element = document.createElement('script');
    element.src = args.url;
    element.type = 'text/javascript';
    element.async = true;
    element.onload = () => {
      console.log(`Dynamic Script Loaded: ${args.url}`);
      cacheScript[args.url] = true;
      resolve(true);
    };

    element.onerror = () => {
      console.error(`Dynamic Script Error: ${args.url}`);
      resolve(false);
    };
    document.head.appendChild(element);
    //document.head.removeChild(element);
  });
};

const loaderRouter = async options => {
  const isLoad = await useDynamicScript({
    url: options && options.url,
  });

  if (isLoad === false) {
    console.log(`Failed to load dynamic script: ${options.url}`);
    return;
  }

  loadComponent(options.scope, options.module)().then(routers => {
    const container = document.getElementById('magic-master-federation-container');
    let router;

    routers.map(item => {
      if (item.path === options.pathname) {
        router = item;
      }
    });

    if (router) {
      ReactDom.render(<Markdown mdx={router.mdx} />, container);
    }
  });
};

// setSystem({
//   url: "http://127.0.0.1:8000/tf-react-upload/remoteEntry.js",
//   scope: "tf_react_upload",
//   module: "./routes",
// });
const buildContent = (pathname, component) => {
  const currentConponent = component[pathname];
  if (!(currentConponent && currentConponent.isFederetion)) {
    return;
  }

  const host = 'http://127.0.0.1:8000';
  const option = {
    url: `${host}${currentConponent.appName}remoteEntry.js`,
    scope: currentConponent.appName.replace(/\//g, '').replace(/\-/g, '_'),
    module: './routes',
    pathname,
  };
  loaderRouter(option);
};

export default buildContent;
